@@ -9,6 +9,8 @@ from kolibri.core.workflow import Workflow, WorkflowIsUserExlcusive, \
     WorkflowCannotProcessQuerysetUsingDifferentModel, WorkflowCannotProcessQuerysetWithoutSettingAModel
 from kolibri.models import *
 
+Context._override_async_for_testing = True
+
 
 class Article(models.Model):
     title = models.CharField(max_length=128)
@@ -27,9 +29,13 @@ dirty_words = ('shiltz', 'fudge',)
 class ProfanityRemover(Processor):
     user_exclusive = True
     model = Article
+    only_run_once = True
     _text = "Remover of profanity"
 
     def process(self, user, article, **kwargs):
+        if u'raise' in article.text:
+            raise ValueError("Testing")
+
         if article.id == 4:
             raise ValueError("Exception raised from %s." % self.__class__.__name__)
         
@@ -444,13 +450,13 @@ class testKolibriBasics(unittest.TestCase):
         trouble_maker = Article.objects.create(title='This will raise exception', text='raise', author=self.joe)
         article_count = Article.objects.all().count()
         workflow = Workflow('Publish article')
-        workflow.first(pr).on_exception(ValueError, pa).on_exception(ValueError, dar)
+        workflow.first(pr).on_exception(ValueError, dar)
         ctx = Context()
         ctx.register.workflow(workflow)
-        ctx.execute.workflow.one(self.joe, workflow.identifier(), trouble_maker)
 
+        ctx.execute.workflow.one(self.joe, workflow.identifier(), trouble_maker)
         self.assertEqual(ProcessItem.objects.all().count(), 1)
-        self.assertEqual(ProcessException.objects.all().count(), 2)
+        self.assertEqual(ProcessException.objects.all().count(), 1)
         self.assertTrue(Article.objects.all().count() < article_count)
 
     def test_refactoring_models_using_then(self):
@@ -544,6 +550,16 @@ class testKolibriBasics(unittest.TestCase):
         self.assertTrue(pr.text == "Remover of profanity")
         self.assertTrue(pr._text == "Remover of profanity")
 
+    def test_only_run_once(self):
+        workflow = Workflow('Publish article')
+        workflow.first(pr).then(pr)
+        ctx = Context()
+        ctx.register.workflow(workflow)
+        ctx.execute.workflow.one(self.joe, workflow.identifier(), self.dirty1)
+        self.assertTrue(ProcessItem.objects.all().count() == 2)
+        self.assertTrue(ProcessItem.objects.all()[1].result == ProcessResult.allready_processed)
+
+
 if __name__ == '__main__':
     suite = unittest.TestLoader().loadTestsFromTestCase(testKolibriBasics)
     unittest.TextTestRunner(verbosity=2).run(suite)
\ No newline at end of file
